@@ -52,8 +52,6 @@ extern int sc_pkcs15emu_atrust_acos_init_ex(sc_pkcs15_card_t *p15card,
 					sc_pkcs15emu_opt_t *opts);
 extern int sc_pkcs15emu_tccardos_init_ex(sc_pkcs15_card_t *, sc_pkcs15emu_opt_t *);
 
-extern int sc_pkcs15emu_rutoken_init_ex(sc_pkcs15_card_t *, sc_pkcs15emu_opt_t *);
-
 extern int sc_pkcs15emu_entersafe_init_ex(sc_pkcs15_card_t *, sc_pkcs15emu_opt_t *);
 
 static struct {
@@ -72,7 +70,6 @@ static struct {
 	{ "actalis",	sc_pkcs15emu_actalis_init_ex	},
 	{ "atrust-acos",sc_pkcs15emu_atrust_acos_init_ex},
 	{ "tccardos",	sc_pkcs15emu_tccardos_init_ex	},
- 	{ "rutoken",	sc_pkcs15emu_rutoken_init_ex 	},
 	{ "entersafe",  sc_pkcs15emu_entersafe_init_ex  },
 	{ NULL, NULL }
 };
@@ -108,17 +105,11 @@ sc_pkcs15_bind_synthetic(sc_pkcs15_card_t *p15card)
 	memset(&opts, 0, sizeof(opts));
 	conf_block = NULL;
 
-	for (i = 0; ctx->conf_blocks[i] != NULL; i++) {
-		blocks = scconf_find_blocks(ctx->conf, ctx->conf_blocks[i],
-						"framework", "pkcs15");
-		if (blocks && blocks[0] != NULL)
-			conf_block = blocks[0];
-		free(blocks);
-	}
+	conf_block = sc_get_conf_block(ctx, "framework", "pkcs15", 1);
 
 	if (!conf_block) {
-		/* no conf file found => try the internal drivers  */
-		sc_debug(ctx, "no conf file, trying builtin emulators\n");
+		/* no conf file found => try bultin drivers  */
+		sc_debug(ctx, "no conf file (or section), trying all builtin emulators\n");
 		for (i = 0; builtin_emulators[i].name; i++) {
 			sc_debug(ctx, "trying %s\n", builtin_emulators[i].name);
 			r = builtin_emulators[i].handler(p15card, &opts);
@@ -128,16 +119,16 @@ sc_pkcs15_bind_synthetic(sc_pkcs15_card_t *p15card)
 		}
 	} else {
 		/* we have a conf file => let's use it */
+		int builtin_enabled; 
 		const scconf_list *list, *item;
 
-		/* find out if the internal drivers should be used */
-		i = scconf_get_bool(conf_block, "enable_builtin_emulation", 1);
-		if (i) {
-			/* get the list of the internal drivers */
-			sc_debug(ctx, "use builtin drivers\n");
-			list = scconf_find_list(conf_block, "builtin_emulators");
+		builtin_enabled = scconf_get_bool(conf_block, "enable_builtin_emulation", 1);
+		list = scconf_find_list(conf_block, "builtin_emulators"); /* FIXME: rename to enabled_emulators */
+
+		if (builtin_enabled && list) {
+			/* get the list of enabled emulation drivers */
 			for (item = list; item; item = item->next) {
-				/* get through the list of builtin drivers */
+				/* go through the list of builtin drivers */
 				const char *name = item->data;
 
 				sc_debug(ctx, "trying %s\n", name);
@@ -145,8 +136,19 @@ sc_pkcs15_bind_synthetic(sc_pkcs15_card_t *p15card)
 					if (!strcmp(builtin_emulators[i].name, name)) {
 						r = builtin_emulators[i].handler(p15card, &opts);
 						if (r == SC_SUCCESS)
+							/* we got a hit */
 							goto out;
 					}
+			}	
+		}
+		if (builtin_enabled) {
+			sc_debug(ctx, "no emulator list in config file, trying all builtin emulators\n");
+			for (i = 0; builtin_emulators[i].name; i++) {
+				sc_debug(ctx, "trying %s\n", builtin_emulators[i].name);
+				r = builtin_emulators[i].handler(p15card, &opts);
+				if (r == SC_SUCCESS)
+					/* we got a hit */
+					goto out;
 			}
 		}
 
@@ -171,6 +173,7 @@ sc_pkcs15_bind_synthetic(sc_pkcs15_card_t *p15card)
 
 out:	if (r == SC_SUCCESS) {
 		p15card->magic  = SC_PKCS15_CARD_MAGIC;
+		p15card->flags |= SC_PKCS15_CARD_FLAG_EMULATED;
 	} else if (r != SC_ERROR_WRONG_CARD) {
 		sc_error(ctx, "Failed to load card emulator: %s\n",
 				sc_strerror(r));
