@@ -8,7 +8,7 @@
 #include <string.h>
 #include "sc-pkcs11.h"
 
-#ifdef HAVE_OPENSSL
+#ifdef ENABLE_OPENSSL
 #include <openssl/evp.h>
 #include <openssl/rand.h>
 #include <openssl/rsa.h>
@@ -136,9 +136,9 @@ static CK_RV sc_pkcs11_openssl_md_final(sc_pkcs11_operation_t *op,
 				CK_BYTE_PTR pDigest, CK_ULONG_PTR pulDigestLen)
 {
 	EVP_MD_CTX	*md_ctx = DIGEST_CTX(op);
-	unsigned int	len = *pulDigestLen;
+	CK_ULONG	len = *pulDigestLen;
 
-	if (len < EVP_MD_CTX_size(md_ctx)) {
+	if (len < (CK_ULONG)EVP_MD_CTX_size(md_ctx)) {
 		*pulDigestLen = EVP_MD_CTX_size(md_ctx);
 		return CKR_BUFFER_TOO_SMALL;
 	}
