@@ -32,7 +32,7 @@
 sc_context_t *context = NULL;
 struct sc_pkcs11_pool session_pool;
 struct sc_pkcs11_slot *virtual_slots = NULL;
-struct sc_pkcs11_card card_table[SC_PKCS11_MAX_READERS];
+struct sc_pkcs11_card card_table[SC_MAX_READERS];
 struct sc_pkcs11_config sc_pkcs11_conf;
 #if !defined(_WIN32)
 pid_t initialized_pid = (pid_t)-1;
@@ -212,16 +212,16 @@ CK_RV C_Initialize(CK_VOID_PTR pInitArgs)
 
 	first_free_slot = 0;
 	virtual_slots = (struct sc_pkcs11_slot *)malloc(
-		sizeof (*virtual_slots) * sc_pkcs11_conf.pkcs11_max_virtual_slots
+		sizeof (*virtual_slots) * sc_pkcs11_conf.max_virtual_slots
 	);
 	if (virtual_slots == NULL) {
 		rv = CKR_HOST_MEMORY;
 		goto out;
 	}
 	pool_initialize(&session_pool, POOL_TYPE_SESSION);
-	for (i=0; i<sc_pkcs11_conf.pkcs11_max_virtual_slots; i++)
+	for (i=0; i<sc_pkcs11_conf.max_virtual_slots; i++)
 		slot_initialize(i, &virtual_slots[i]);
-	for (i=0; i<SC_PKCS11_MAX_READERS; i++)
+	for (i=0; i<SC_MAX_READERS; i++)
 		card_initialize(i);
 
 	/* Detect any card, but do not flag "insert" events */
@@ -295,15 +295,15 @@ CK_RV C_GetInfo(CK_INFO_PTR pInfo)
 
 	memset(pInfo, 0, sizeof(CK_INFO));
 	pInfo->cryptokiVersion.major = 2;
-	pInfo->cryptokiVersion.minor = 11;
+	pInfo->cryptokiVersion.minor = 20;
 	strcpy_bp(pInfo->manufacturerID,
 		  "OpenSC (www.opensc-project.org)",
 		  sizeof(pInfo->manufacturerID));
 	strcpy_bp(pInfo->libraryDescription,
 		  "smart card PKCS#11 API",
 		  sizeof(pInfo->libraryDescription));
-	pInfo->libraryVersion.major = 1;
-	pInfo->libraryVersion.minor = 0;
+	pInfo->libraryVersion.major = 0;
+	pInfo->libraryVersion.minor = 0; /* FIXME: use 0.116 for 0.11.6 from autoconf */
 
 out:	sc_pkcs11_unlock();
 	return rv;
@@ -339,7 +339,7 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 
 	if (
 		(found = (CK_SLOT_ID_PTR)malloc (
-			sizeof (*found) * sc_pkcs11_conf.pkcs11_max_virtual_slots
+			sizeof (*found) * sc_pkcs11_conf.max_virtual_slots
 		)) == NULL
 	) {
 		rv = CKR_HOST_MEMORY;
@@ -347,13 +347,14 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 	}
 
 	sc_debug(context, "Getting slot listing\n");
-	if (pSlotList == NULL_PTR) {
+	/* Slot list can only change in v2.20 */
+	if (pSlotList == NULL_PTR && sc_pkcs11_conf.plug_and_play) {
 		sc_ctx_detect_readers(context);
 	}
 	card_detect_all();
 
 	numMatches = 0;
-	for (i=0; i<sc_pkcs11_conf.pkcs11_max_virtual_slots; i++) {
+	for (i=0; i<sc_pkcs11_conf.max_virtual_slots; i++) {
 		slot = &virtual_slots[i];
 
 		if (!tokenPresent || (slot->slot_info.flags & CKF_TOKEN_PRESENT))
@@ -684,7 +685,7 @@ sc_pkcs11_init_lock(CK_C_INITIALIZE_ARGS_PTR args)
 		/* Shall be used in threaded envirnoment, must use app provided locking */
 		global_locking = args;
 	} else if (!applock && !oslock) {
-		/* Shall not be used in threaded environemtn, use operating system locking */
+		/* Shall not be used in threaded environment, use operating system locking */
 		global_locking = default_mutex_funcs;
 	}
 
@@ -753,7 +754,7 @@ void sc_pkcs11_free_lock(void)
 }
 
 CK_FUNCTION_LIST pkcs11_function_list = {
-	{ 2, 11 },
+	{ 2, 11 }, /* Note: NSS/Firefox ignores this version number and uses C_GetInfo() */
 	C_Initialize,
 	C_Finalize,
 	C_GetInfo,
